library benchmarks.src.costs.index;

import "package:angular2/angular2.dart"
    show
        bootstrap,
        Component,
        Decorator,
        View,
        DynamicComponentLoader,
        ElementRef,
        DynamicComponent;
import "package:angular2/src/core/life_cycle/life_cycle.dart" show LifeCycle;
import "package:angular2/src/facade/collection.dart" show List, ListWrapper;
import "package:angular2/src/reflection/reflection.dart" show reflector;
import "package:angular2/src/reflection/reflection_capabilities.dart"
    show ReflectionCapabilities;
import "package:angular2/src/test_lib/benchmark_util.dart"
    show getIntParameter, bindAction;
import "package:angular2/directives.dart" show If, For;

var testList = null;
main() {
  reflector.reflectionCapabilities = new ReflectionCapabilities();
  var size = getIntParameter("size");
  testList = ListWrapper.createFixedSize(size);
  bootstrap(AppComponent).then((ref) {
    var injector = ref.injector;
    AppComponent app = injector.get(AppComponent);
    var lifeCycle = injector.get(LifeCycle);
    bindAction("#reset", () {
      app.reset();
      lifeCycle.tick();
    }); // Baseline (plain components)
    bindAction("#createPlainComponents", () {
      app.createPlainComponents();
      lifeCycle.tick();
    }); // Components with decorators
    bindAction("#createComponentsWithDecorators", () {
      app.createComponentsWithDecorators();
      lifeCycle.tick();
    }); // Components with decorators
    bindAction("#createDynamicComponents", () {
      app.createDynamicComponents();
      lifeCycle.tick();
    });
  });
}
@Component(selector: "app")
@View(
    directives: const [If, For, DummyComponent, DummyDecorator, DynamicDummy],
    template: '''
    <div *if="testingPlainComponents">
      <dummy *for="#i of list"></dummy>
    </div>

    <div *if="testingWithDecorators">
      <dummy dummy-decorator *for="#i of list"></dummy>
    </div>

    <div *if="testingDynamicComponents">
      <dynamic-dummy *for="#i of list"></dynamic-dummy>
    </div>
  ''')
class AppComponent {
  List list;
  bool testingPlainComponents;
  bool testingWithDecorators;
  bool testingDynamicComponents;
  AppComponent() {
    this.reset();
  }
  void reset() {
    this.list = [];
    this.testingPlainComponents = false;
    this.testingWithDecorators = false;
    this.testingDynamicComponents = false;
  }
  void createPlainComponents() {
    this.list = testList;
    this.testingPlainComponents = true;
  }
  void createComponentsWithDecorators() {
    this.list = testList;
    this.testingWithDecorators = true;
  }
  void createDynamicComponents() {
    this.list = testList;
    this.testingDynamicComponents = true;
  }
}
@Component(selector: "dummy")
@View(template: '''<div></div>''')
class DummyComponent {}
@Decorator(selector: "[dummy-decorator]")
class DummyDecorator {}
@DynamicComponent(selector: "dynamic-dummy")
class DynamicDummy {
  DynamicDummy(DynamicComponentLoader loader, ElementRef location) {
    loader.loadIntoExistingLocation(DummyComponent, location);
  }
}
